'use strict';

const util = require('util');
const Stream = require('stream').Stream;

// 日志配置
const cfg = {
    time: false,                // 记录附带时间 
    ua: false,                  // 记录user-agent
    ip: false,                  // 记录ip
    req_header: false,          // 记录请求头
    req_body: false,            // 记录请求主体
    res_body: false,            // 记录返回体
    sql_result: false,          // 记录sql的返回数据
    fetch_result: false         // 记录fetch的返回数据
}

// 输出时间
const time = function () {
    const d = new Date();
    return `${d.getFullYear()}-${d.getMonth() + 1}-${d.getDate()} ${d.getHours()}:${d.getMinutes()}:${d.getSeconds()}.${d.getMilliseconds()}`;
}

// 修改配置
const buildCfg = function (para) {
    const allTrue = para.indexOf('-true-') > -1;
    const allFalse = para.indexOf('-false-') > -1;

    for (const k in cfg) {
        if (allTrue) {
            cfg[k] = true;
        } else if (allFalse) {
            cfg[k] = false;
        } else if (para.indexOf(`-${k}-`) > -1) {
            cfg[k] = true;
        }
    }
}

// 输入日志
const out = function (ctx, name, para) {
    const record = `--- ${ctx.rid || 0}: ${cfg.time ? time() : ''} [${name}] ${para}`;
    // 输出到控制台
    console.log(record);
    // 记录到文件
    // ……
}

// 处理各类日志
const log = {

    // 请求
    req(ctx) {

        // 修改配置
        ctx.query.rtlog && buildCfg(ctx.query.rtlog);

        // 输出请求概要：方式、地址
        out(ctx, ctx.method, ctx.url);

        // 配置输出user-agent
        cfg.ua && out(ctx, 'ua', ctx.req.headers['user-agent']);

        // 配置输出ip
        cfg.ip && out(ctx, 'ip', ctx.ip);

        // 配置输出请求主体
        cfg.req_header && out(ctx, 'req_header', JSON.stringify(ctx.req.headers));
    },

    // 响应
    res(ctx) {

        // 配置输出请求体
        cfg.req_body && out(ctx, 'req_body', JSON.stringify(ctx.request.body));

        // 输出响应概要：状态、长度
        out(ctx, ctx.status, ctx.length);

        // 配置输出请求内容
        if (cfg.res_body) {
            const body = ctx.body;
            if (body instanceof Stream) {
                out(ctx, 'res_body', '[Stream]');
            } else if (ctx.response.type.indexOf('/json') > 0) {
                out(ctx, 'res_body', JSON.stringify(body));
            } else {
                out(ctx, 'res_body', util.inspect(body));
            }
        };
    },

    // 执行sql
    sql(ctx, sql, paras) {
        // 输出请求的sql和参数表
        out(ctx, 'sql', sql + ' | ' + JSON.stringify(paras));
    },

    // 执行结果
    sql_done(ctx, err, result) {
        if (err) {
            // 输出sql错误
            out(ctx, 'sql-error', util.inspect(err));
        } else if (cfg.sql_result) {
            // 配置输出sql结果
            out(ctx, 'sql-result', JSON.stringify(result));
        }
    },

    // 异步请求
    fetch(ctx, url) {
        // 输出请求的地址
        out(ctx, 'fetch', url);
    },

    // 异步请求结果
    fetch_done(ctx, err, res) {
        if (err) {
            // 输出请求错误
            out(ctx, 'fetch-error', util.inspect(err));
        } else {
            // 输出请求结果概要：状态
            out(ctx, 'fetch-done', res.statusCode);
            // 配置输出请求结果
            cfg.fetch_result && out(ctx, 'fetch_result', res.body);
        }
    }
}

module.exports = function (name, ...args) {

    const ctx = this || { rid: '_' };

    if (name in log) {
        log[name](ctx, ...args);
    } else {
        out(ctx, name, util.inspect(args));
    }
}