
const ApiBase = require('./apiBase');

// 生成查询参数
class SearchOpt {

    constructor(body, page) {
        const t = this;

        t.order = body.sortField + (body.sortDesc ? ' desc' : ' asc');
        t.pageIndex = body.pagerIndex;
        t.pageSize = body.pagerSize;
        t.paras = [];

        t._page = page;
        t._where = [];
        t._condition = body.condition;
    }


    get where() {
        return this._where.join(' and ');
    }


    // 设置搜索条件
    setCondition(key, exp, para) {
        const t = this;
        const v = t._condition[key];
        if (!v) { return t; }

        const val = t._page.value(v, para, `参数${key}输入有误`);
        t._where.push(exp);
        t.paras.push(val);

        return t;
    }


    async search(access) {
        const t = this;
        const r = await access
            .select('*', t.where, t.order)
            .getPage(t.paras, t.pageSize, t.pageIndex);
        return { items: r.rows, total: r.total }
    }
}

// 基类
module.exports = class extends ApiBase {

    /** 返回成功信息 */
    output(data) {
        return {
            result: data,
            status: 'SUCCESS',
            status_code: 0,
            status_msg: '操作成功'
        }
    }

    /** 生成查询配置 */
    searchOpt(body) {
        return new SearchOpt(body, this);
    }

    /** 获取概要 */
    async $schema(body) {
        const t = this;
        const r = await t.buildSchema(body);
        return t.output(r);
    }

    /** 搜索记录集 */
    async $search(body) {
        const t = this;
        const o = await t.buildSearcher(body);

        if (t.isFailed) { return t.fail(); }

        const r = await o.search(t.access);
        await t.access.buildOuter(r.items);

        return t.output(r);
    }

    /** 查找一条记录 */
    async $find(body) {
        const t = this;
        const id = t.value(body.id, 0, '参数id必须为数字');
        if (t.isFailed) { return t.fail(); }

        const pk = t.access.primaryKey;
        const r = await t.access.select('*', `${pk}=?`).getOne(id);
        await t.access.buildOuter(r);
        
        return t.output(r);
    }

    /** 创建一条记录 */
    async $create(body) {
        const t = this;
        const data = await t.buildData(body, 'create');
        if (t.isFailed) { return t.fail(); }

        const r = await t.access.insert(data);
        return t.output(r);
    }

    /** 更新一条记录 */
    async $update(body) {
        const t = this;
        const pk = t.access.primaryKey;
        const id = t.value(body[pk], 0, '参数id必须为数字');
        const data = await t.buildData(body, 'update');
        if (t.isFailed) { return t.fail(); }

        const r = await t.access.update(data, { [pk]: id });
        return t.output(r);
    }

    /** 移除一条记录 */
    async $remove(body) {
        const t = this;
        const pk = t.access.primaryKey;
        const ids = t.value(body.ids, /^\d+(\,\d+)*$/, 'ids格式有误');
        if (t.isFailed) { return t.fail(); }

        const r = await t.access.delete(`${pk} in (${ids})`);
        return t.output(r.affectedRows);
    }

    Schema(body) { return this.$schema(body); }
    Search(body) { return this.$search(body); }
    Find(body) { return this.$find(body); }
    Create(body) { return this.$create(body); }
    Update(body) { return this.$update(body); }
    Remove(body) { return this.$remove(body); }
}