const mysql = require('mysql');
const Select = require('./select');

class Access {

    constructor(db_cfg, acc_cfg) {
        const t = this;
        t.db_cfg = db_cfg;                              // 数据库配置
        t.ctx = db_cfg._ctx || null;                    // 上下文

        if (acc_cfg) {
            t.sheetName = acc_cfg.sheetName;                // 表名
            t.primaryKey = acc_cfg.primaryKey || 'id';      // 唯一主键
            t.insertData = acc_cfg.insertData;              // 插入数据的默认值
            t.filter = acc_cfg.filter;                      // 查询的默认过滤条件
            t.outerExpress = acc_cfg.outerExpress || {};    // 外键描述
        }
    }

    // 设置上下文
    setCtx(ctx) {
        return (this.ctx = ctx) && this;
    }

    // #region 构建表名

    /** 设置表名和主键 */
    setEntity(sheetName, primaryKey = 'id') {
        const t = this;
        t.sheetName = sheetName;
        t.primaryKey = primaryKey;
        return t;
    }

    /** 追加表名 */
    addEntity(express) {
        const t = this;
        t.sheetName += ' ' + express;
        return t;
    }

    // #endregion

    // #region 执行操作

    /** 开启事务 */
    async beginTransaction() {
        const t = this;
        t.db_cfg._connection = null;

        return new Promise((res, rej) => {
            const pool = t.db_cfg._pool = t.db_cfg._pool || mysql.createPool(t.db_cfg);
            pool.getConnection(function (err1, connection) {
                if (err1) { return rej(err1) }

                t.db_cfg._connection = connection;
                connection.beginTransaction(err2 => {
                    if (err2) { return rej(err2); }
                    res();
                });
            });
        });
    }

    /** 提交事务 */
    async commit() {
        const t = this;
        return new Promise((res, rej) => {

            let connection = t.db_cfg._connection;
            connection.commit(err => {
                if (err) {
                    connection.rollback(() => rej(err));
                } else {
                    res();
                    connection.release();
                    t.db_cfg._connection = connection = null;
                }
            });
        });
    }

    /** 执行sql语句 */
    async query(sql, paras) {
        const t = this;

        // 将参数转为数组
        if (paras != undefined && !Array.isArray(paras)) {
            paras = [paras];
        }

        // 日志
        const log = ((t.ctx && t.ctx.rlog) || function () { }).bind(t.ctx);
        log('sql', sql, paras);

        return new Promise((res, rej) => {

            // 事务
            if (t.db_cfg._connection) {
                const connection = t.db_cfg._connection;
                connection.query(sql, paras, function (error, results, fields) {
                    log('sql_done', error, results);
                    error ? rej(error) : res(results);
                });
                return;
            }

            // 启用连接池
            const pool = t.db_cfg._pool = t.db_cfg._pool || mysql.createPool(t.db_cfg);

            pool.getConnection(function (err, connection) {
                err && log('sql_done', err, null);

                connection.query(sql, paras, function (error, results, fields) {
                    connection.release();
                    log('sql_done', error, results);
                    error ? rej(error) : res(results);
                });
            });
        });
    }

    /** 构造查询 */
    select(field, where, order) {
        const t = this;
        const select = new Select(t, t.filter);

        field && select.field(field);
        where && select.where(where);
        order && select.order(order);

        return select;
    }

    /** 不依赖过滤器的构造查询 */
    sysSelect(field, where, order) {
        const t = this;
        const select = new Select(t);

        field && select.field(field);
        where && select.where(where);
        order && select.order(order);

        return select;
    }

    /** 添加数据 */
    async insert(paras) {
        const t = this;
        if (t.insertData) {
            paras = Object.assign({}, t.insertData, paras);
        }
        const cols = [];
        const vals = [];
        const arr = [];

        for (let key in paras) {
            cols.push('`' + key + '`');
            vals.push('?');
            arr.push(paras[key]);
        }

        const sql = `insert into ${t.sheetName} (${cols.join(',')}) values (${vals.join(',')})`;

        return await t.query(sql, arr);
    }

    /** 更新数据 */
    async update(paras, condition) {
        const t = this;
        const cols = [];
        const arr = [];

        for (let key in paras) {
            cols.push('`' + key + '`=?');
            arr.push(paras[key]);
        }

        // Object类型的条件
        if (typeof condition !== 'string') {
            const tmp = [];
            for (let key in condition) {
                tmp.push(`${key}=?`);
                arr.push(condition[key]);
            }
            condition = tmp.join(' and ');
        }

        const sql = `update ${t.sheetName} set ${cols.join(',')} where ${condition}`;
        return await t.query(sql, arr);
    }

    /** 删除数据 */
    async delete(condition) {
        const t = this;
        const arr = [];

        // Object类型的条件
        if (typeof condition !== 'string') {
            const tmp = [];
            for (let key in condition) {
                tmp.push(`${key}=?`);
                arr.push(condition[key]);
            }
            condition = tmp.join(' and ');
        }

        const sql = `delete from ${t.sheetName} where ${condition}`
        return await t.query(sql, arr);
    }

    /** 构建外部数据 */
    async buildOuter(rows, keys) {
        const t = this;

        if (!rows) { return rows; }
        const isArray = Array.isArray(rows)
        if (!isArray) { rows = [rows]; }

        if (rows.length > 0) {
            // 如果没有指定key
            if (!keys) {
                keys = [];
                for (const k in t.outerExpress) {
                    keys.push(k);
                }
            }

            // 构建内容
            for (const key of keys) {
                const [, sheet, col] = t.outerExpress[key].split(/\#|\./g);

                const v_arr = [];
                for (const row of rows) {
                    const v = row[key];
                    if (v !== null && !v_arr.includes(v)) {
                        v_arr.push(v);
                    }
                    row[key + '__data'] = null;
                }

                if (v_arr.length > 0) {
                    const access = new Access(t.db_cfg, { sheetName: sheet });
                    const arr = await access.sysSelect().where(`${col} in ('${v_arr.join("','")}')`).getList();
                    for (const row of rows) {
                        const r = arr.find(r => r[col] == row[key]);
                        row[key + '__data'] = r === undefined ? null : r;
                    }
                }
            }
        }

        return isArray ? rows : rows[0];
    }

    /** 构建外部数据，优化版 */
    buildOuter2(rows, keys) {
        const t = this;
        return new Promise((res, rej) => {

            if (!rows) {
                res(rows);
                return;
            }

            const isArray = Array.isArray(rows)
            if (!isArray) { rows = [rows]; }

            let task_count = 0;

            if (rows.length > 0) {
                // 如果没有指定key
                if (!keys) {
                    keys = [];
                    for (const k in t.outerExpress) {
                        keys.push(k);
                    }
                }

                // 构建内容
                for (const key of keys) {
                    const [, sheet, col] = t.outerExpress[key].split(/\#|\./g);
                    const access = new Access(t.db_cfg, { sheetName: sheet });

                    for (const row of rows) {
                        const v = row[key];
                        if (v !== null) {
                            task_count++;
                            access.sysSelect().where(`${col}=?`).limit(1).getOne(v).then(v => {
                                row[key + '__data'] = v;
                                task_count--
                                if (task_count == 0) {
                                    res(isArray ? rows : rows[0]);
                                }
                            })
                        } else {
                            row[key + '__data'] = null;
                        }
                    }
                }
            }

            if (task_count == 0) {
                res(isArray ? rows : rows[0]);
            }
        });
    }

    // #endregion

    /** 附加数据 */
    static async withItem(key, list, outerKey, access, where) {

        let count = 0;

        const fn = async (item) => {
            const result = await access.sysSelect().where(where).getOne(item[outerKey]);
            item[key] = result;
            if (result != null) { count++; }
        }

        if (list instanceof Array) {
            for (const item of list) { await fn(item); }
        } else if (list) {
            await fn(list);
        }

        return count;
    }
}

module.exports = Access;